@@ -182,7 +182,11 @@ static char *lookup_ssl_cert_dn(X509_NAME *xsname, int dnidx)
                     ASN1_STRING *adata = X509_NAME_ENTRY_get_data(xsne);
                     int len = ASN1_STRING_length(adata);
                     result = malloc(len + 1);
+#if defined(LIBRESSL_VERSION_NUMBER)
                     memcpy(result, ASN1_STRING_data(adata), len);
+#else
+                    memcpy(result, ASN1_STRING_get0_data(adata), len);
+#endif
                     result[len] = '\0';
 
 #if APR_CHARSET_EBCDIC
@@ -394,11 +398,15 @@ TCN_IMPLEMENT_CALL(jstring, SSLSocket, getInfoS)(TCN_STDARGS, jlong sock,
                     }
                 break;
                 case SSL_INFO_CLIENT_A_SIG:
+#if defined(LIBRESSL_VERSION_NUMBER)
+                	value = tcn_new_string(e, "UNKNOWN");
+#else
                     nid = X509_get_signature_nid(xs);
                     if (nid == NID_undef)
                         value = tcn_new_string(e, "UNKNOWN");
                     else
                         value = tcn_new_string(e, OBJ_nid2ln(nid));
+#endif
                 break;
                 case SSL_INFO_CLIENT_A_KEY:
                     X509_PUBKEY_get0_param(&paobj, NULL, 0, NULL, X509_get_X509_PUBKEY(xs));
@@ -445,11 +453,15 @@ TCN_IMPLEMENT_CALL(jstring, SSLSocket, getInfoS)(TCN_STDARGS, jlong sock,
                     }
                 break;
                 case SSL_INFO_SERVER_A_SIG:
+#if defined(LIBRESSL_VERSION_NUMBER)
+                	value = tcn_new_string(e, "UNKNOWN");
+#else
                     nid = X509_get_signature_nid(xs);
                     if (nid == NID_undef)
                         value = tcn_new_string(e, "UNKNOWN");
                     else
                         value = tcn_new_string(e, OBJ_nid2ln(nid));
+#endif
                 break;
                 case SSL_INFO_SERVER_A_KEY:
                     X509_PUBKEY_get0_param(&paobj, NULL, 0, NULL, X509_get_X509_PUBKEY(xs));
