@@ -5,7 +5,7 @@
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *     https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -166,26 +166,24 @@ static int set_user_group(const char *user, int uid, int gid)
 
 /* Set linux capability, user and group */
 #ifdef OS_LINUX
-/* CAPSALL is to allow to read/write at any location */
-#define LEGACY_CAPSALL  (1 << CAP_NET_BIND_SERVICE) +   \
-                        (1 << CAP_SETUID) +             \
-                        (1 << CAP_SETGID) +             \
-                        (1 << CAP_DAC_READ_SEARCH) +    \
-                        (1 << CAP_DAC_OVERRIDE)
 
-#define LEGACY_CAPSMAX  (1 << CAP_NET_BIND_SERVICE) +   \
-                        (1 << CAP_DAC_READ_SEARCH) +    \
-                        (1 << CAP_DAC_OVERRIDE)
+/* Preferred capabilities for getuid/setuid */
+#define LEGACY_CAPS_UID         (1 << CAP_NET_BIND_SERVICE) +   \
+                                (1 << CAP_DAC_READ_SEARCH) +    \
+                                (1 << CAP_SETUID) +             \
+                                (1 << CAP_SETGID)
 
-/* That a more reasonable configuration */
-#define LEGACY_CAPS     (1 << CAP_NET_BIND_SERVICE) +   \
-                        (1 << CAP_DAC_READ_SEARCH) +    \
-                        (1 << CAP_SETUID) +             \
-                        (1 << CAP_SETGID)
+/* Minimum required capabilities for getuid/setuid */
+#define LEGACY_CAPS_UID_MIN     (1 << CAP_NET_BIND_SERVICE) +   \
+                                (1 << CAP_SETUID) +             \
+                                (1 << CAP_SETGID)
 
-/* probably the only one Java could use */
-#define LEGACY_CAPSMIN  (1 << CAP_NET_BIND_SERVICE) +   \
-                        (1 << CAP_DAC_READ_SEARCH)
+/* Preferred capabilities without getuid/setuid */
+#define LEGACY_CAPS_NO_UID      (1 << CAP_NET_BIND_SERVICE) +   \
+                                (1 << CAP_DAC_READ_SEARCH)
+
+/* Minimum required capabilities without getuid/setuid */
+#define LEGACY_CAPS_NO_UID_MIN  (1 << CAP_NET_BIND_SERVICE)
 
 #define LEGACY_CAP_VERSION  0x19980330
 static int set_legacy_caps(int caps)
@@ -209,20 +207,32 @@ static int set_legacy_caps(int caps)
 }
 
 #ifdef HAVE_LIBCAP
-static cap_value_t caps_std[] = {
+static cap_value_t caps_uid[] = {
     CAP_NET_BIND_SERVICE,
     CAP_SETUID,
     CAP_SETGID,
     CAP_DAC_READ_SEARCH
 };
 
-static cap_value_t caps_min[] = {
+static cap_value_t caps_uid_min[] = {
+    CAP_NET_BIND_SERVICE,
+    CAP_SETUID,
+    CAP_SETGID
+};
+
+static cap_value_t caps_no_uid[] = {
     CAP_NET_BIND_SERVICE,
     CAP_DAC_READ_SEARCH
 };
 
-#define CAPS     1
-#define CAPSMIN  2
+static cap_value_t caps_no_uid_min[] = {
+    CAP_NET_BIND_SERVICE
+};
+
+#define CAPS_UID        1
+#define CAPS_UID_MIN    2
+#define CAPS_NO_UID     3
+#define CAPS_NO_UID_MIN 4
 
 
 typedef int (*fd_cap_free) (void *);
@@ -301,25 +311,29 @@ static int set_caps(int cap_type)
     int ncap;
     int flag = CAP_SET;
     cap_value_t *caps;
-    const char *type;
 
     if (ld_libcap()) {
         return set_legacy_caps(cap_type);
     }
-    if (cap_type == CAPS) {
-        ncap = sizeof(caps_std) / sizeof(cap_value_t);
-        caps = caps_std;
-        type = "default";
+    if (cap_type == CAPS_UID) {
+        ncap = sizeof(caps_uid) / sizeof(cap_value_t);
+        caps = caps_uid;
     }
-    else if (cap_type == CAPSMIN) {
-        ncap = sizeof(caps_min) / sizeof(cap_value_t);
-        caps = caps_min;
-        type = "min";
+    else if (cap_type == CAPS_UID_MIN) {
+        ncap = sizeof(caps_uid_min) / sizeof(cap_value_t);
+        caps = caps_uid_min;
+    }
+    else if (cap_type == CAPS_NO_UID) {
+        ncap = sizeof(caps_no_uid) / sizeof(cap_value_t);
+        caps = caps_no_uid;
+    }
+    else if (cap_type == CAPS_NO_UID_MIN) {
+        ncap = sizeof(caps_no_uid_min) / sizeof(cap_value_t);
+        caps = caps_no_uid_min;
     }
     else {
-        ncap = sizeof(caps_min) / sizeof(cap_value_t);
-        caps = caps_min;
-        type = "null";
+        ncap = sizeof(caps_no_uid_min) / sizeof(cap_value_t);
+        caps = caps_no_uid_min;
         flag = CAP_CLEAR;
     }
     c = (*fp_cap_init) ();
@@ -328,25 +342,23 @@ static int set_caps(int cap_type)
     (*fp_cap_set_flag) (c, CAP_INHERITABLE, ncap, caps, flag);
     (*fp_cap_set_flag) (c, CAP_PERMITTED, ncap, caps, flag);
     if ((*fp_cap_set_proc) (c) != 0) {
-        log_error("failed setting %s capabilities.", type);
         return -1;
     }
     (*fp_cap_free) (c);
-    if (cap_type == CAPS)
+    if (cap_type == CAPS_UID || cap_type == CAPS_UID_MIN)
         log_debug("increased capability set.");
-    else if (cap_type == CAPSMIN)
-        log_debug("decreased capability set to min required.");
+    else if (cap_type == CAPS_NO_UID || cap_type == CAPS_NO_UID_MIN)
+        log_debug("decreased capability set.");
     else
         log_debug("dropped capabilities.");
     return 0;
 }
 
 #else /* !HAVE_LIBCAP */
-/* CAPSALL is to allow to read/write at any location */
-#define CAPSALL LEGACY_CAPSALL
-#define CAPSMAX LEGACY_CAPSMAX
-#define CAPS    LEGACY_CAPS
-#define CAPSMIN LEGACY_CAPSMIN
+#define CAPS_UID        LEGACY_CAPS_UID
+#define CAPS_UID_MIN    LEGACY_CAPS_UID_MIN
+#define CAPS_NO_UID     LEGACY_CAPS_NO_UID
+#define CAPS_NO_UID_MIN LEGACY_CAPS_NO_UID
 static int set_caps(int caps)
 {
     return set_legacy_caps(caps);
@@ -361,12 +373,15 @@ static int linuxset_user_group(const char *user, int uid, int gid)
         return 0;
     /* set capabilities enough for binding port 80 setuid/getuid */
     if (getuid() == 0) {
-        if (set_caps(CAPS) != 0) {
-            if (getuid() != uid) {
-                log_error("set_caps(CAPS) failed for user '%s'", user);
-                return -1;
+        if (set_caps(CAPS_UID) != 0) {
+            log_debug("set_caps(CAPS_UID) failed for user '%s'", user);
+            if (set_caps(CAPS_UID_MIN) != 0) {
+                if (getuid() != uid) {
+                    log_error("set_caps(CAPS_UID_MIN) failed for user '%s'", user);
+                    return -1;
+                }
+                log_debug("set_caps(CAPS_UID_MIN) failed for user '%s'", user);
             }
-            log_debug("set_caps(CAPS) failed for user '%s'", user);
         }
         /* make sure they are kept after setuid */
         if (prctl(PR_SET_KEEPCAPS, 1, 0, 0, 0) < 0) {
@@ -384,12 +399,15 @@ static int linuxset_user_group(const char *user, int uid, int gid)
 
     if (caps_set) {
         /* set capability to binding port 80 read conf */
-        if (set_caps(CAPSMIN) != 0) {
-            if (getuid() != uid) {
-                log_error("set_caps(CAPSMIN) failed for user '%s'", user);
-                return -1;
+        if (set_caps(CAPS_NO_UID) != 0) {
+            log_debug("set_caps(CAPS_NO_UID) failed for user '%s'", user);
+            if (set_caps(CAPS_NO_UID_MIN) != 0) {
+                if (getuid() != uid) {
+                    log_error("set_caps(CAPS_NO_UID_MIN) failed for user '%s'", user);
+                    return -1;
+                }
+                log_debug("set_caps(CAPS_NO_UID_MIN) failed for user '%s'", user);
             }
-            log_debug("set_caps(CAPSMIN) failed for user '%s'", user);
         }
     }
 
@@ -832,7 +850,7 @@ static int child(arg_data *args, home_data *data, uid_t uid, gid_t gid)
     /* Check wether we need to dump the VM version */
     if (args->vers == true) {
         log_error("jsvc (Apache Commons Daemon) " JSVC_VERSION_STRING);
-        log_error("Copyright (c) 1999-2019 Apache Software Foundation.");
+        log_error("Copyright (c) 1999-2025 Apache Software Foundation.");
         if (java_version() != true) {
             return -1;
         }
@@ -892,7 +910,7 @@ static int child(arg_data *args, home_data *data, uid_t uid, gid_t gid)
     sigaction(SIGUSR2, &act, NULL);
     sigaction(SIGTERM, &act, NULL);
     sigaction(SIGINT, &act, NULL);
-
+    controlled = getpid ();
     log_debug("Waiting for a signal to be delivered");
     create_tmp_file(args);
     while (!stopping) {
@@ -1196,7 +1214,7 @@ int main(int argc, char *argv[])
         /*
          * There is no need to change LD_LIBRARY_PATH
          * if we were not able to find a path to libjvm.so
-         * (additionaly a strdup(NULL) cores dump on my machine).
+         * (additionally a strdup(NULL) cores dump on my machine).
          */
         if (libf != NULL) {
             p1 = strdup(libf);
@@ -1290,7 +1308,7 @@ static int run_controller(arg_data *args, home_data *data, uid_t uid, gid_t gid)
      * These will be replaced in the child process.
      */
     memset(&act, '\0', sizeof(act));
-    act.sa_handler = controller;
+    act.sa_sigaction = controller;
     sigemptyset(&act.sa_mask);
     act.sa_flags = SA_RESTART | SA_NOCLDSTOP | SA_SIGINFO;
 
